@@ -21,10 +21,11 @@ from itertools import izip, chain
 from warnings import warn
 
 from logilab.common.decorators import cached, cachedproperty
+from logilab.common.deprecation import class_deprecated, deprecated
 
 from rql import parse
 
-from cubicweb import Binary
+from cubicweb import Binary, neg_role
 from cubicweb.server.ssplanner import READ_ONLY_RTYPES
 from cubicweb.server.utils import eschema_eid
 
@@ -32,32 +33,46 @@ from cubicweb.schema import VIRTUAL_RTYPES
 from cubicweb.entities import AnyEntity
 from cubicweb.view import EntityAdapter
 
-from cubes.container.utils import (yet_unset,
-                                   ordered_container_etypes,
-                                   container_rtypes_etypes,
-                                   parent_rschemas,
-                                   needs_container_parent,
-                                   _insertmany,
+from cubes.container import (ContainerConfiguration,
+                             parent_rschemas, children_rschemas, parent_eschemas,
+                             _needs_container_parent,
+                             )
+from cubes.container.utils import (_insertmany,
                                    _add_rqlst_restriction,
-                                   _iter_mainvar_relations)
+                                   _iter_mainvar_relations,
+                                   )
+
 
 class Container(AnyEntity):
+    __metaclass__ = class_deprecated
+    __deprecation_warning__ = '[container 2.4] Use ContainerConfiguration'
     __abstract__ = True
 
-    # container API
+    # Deprecated container API (replaced by container_config)
     container_rtype = None
     container_skiprtypes = ()
     container_skipetypes = ()
     container_subcontainers = ()
-    compulsory_hooks_categories = ()
+    compulsory_hooks_categories = () # moved to ContainerClone
+
+    @classmethod
+    def __initialize__(cls, schema):
+        super(Container, cls).__initialize__(schema)
+        cls.container_config = ContainerConfiguration(
+            cls.__regid__, cls.container_rtype,
+            skiprtypes=cls.container_skiprtypes,
+            skipetypes=cls.container_skipetypes,
+            subcontainers=cls.container_subcontainers)
 
 
 @cached
 def container_etypes(vreg):
-    return set(c.__regid__
-               for c, in vreg['etypes'].itervalues()
-               if getattr(c, 'container_rtype', False))
-
+    result = []
+    for regid in vreg['etypes']:
+        eclass = vreg['etypes'].etype_class(regid)
+        if hasattr(eclass, 'container_config'):
+            result.append(regid)
+    return frozenset(result)
 
 @cached
 def first_parent_rtype_role(eschema):
@@ -65,13 +80,14 @@ def first_parent_rtype_role(eschema):
 
 class ContainerProtocol(EntityAdapter):
     __regid__ = 'Container'
-    clone_rtype_role = None
+    __abstract__ = True
 
     @property
-    def related_container(self):
-        if self.entity.e_schema in container_etypes(self._cw.vreg):
-            # self.entity is the container itself
-            return self.entity
+    @deprecated('[container 2.4] Use ContainerConfiguration')
+    def container_rtype(self):
+        # Pre ContainerConfiguration API. Otherwise, the `container_rtype`
+        # attribute is set upon class generation in `build_container_protocol`
+        # method.
         try:
             ccwetype = self.entity.container_etype
         except AttributeError:
@@ -79,8 +95,15 @@ class ContainerProtocol(EntityAdapter):
             return None
         if ccwetype:
             etypes = self._cw.vreg['etypes']
-            crtype = etypes.etype_class(ccwetype[0].name).container_rtype
-            container = getattr(self.entity, crtype, None)
+            return etypes.etype_class(ccwetype[0].name).container_config.rtype
+
+    @property
+    def related_container(self):
+        if self.entity.e_schema in container_etypes(self._cw.vreg):
+            # self.entity is the container itself
+            return self.entity
+        if self.container_rtype is not None:
+            container = getattr(self.entity, self.container_rtype, None)
             if container:
                 return container[0]
         # container relation is still unset, let's ask the parent
@@ -89,12 +112,13 @@ class ContainerProtocol(EntityAdapter):
             container = parent.cw_adapt_to('Container').related_container
             if container is None:
                 return
-            if self.entity.e_schema not in container.container_skipetypes:
+            if (self.entity.e_schema not in
+                    container.container_config.skipetypes):
                 return container
 
     @property
     def parent(self):
-        if needs_container_parent(self.entity.e_schema):
+        if _needs_container_parent(self.entity.e_schema):
             parent = self.entity.container_parent
             return parent[0] if parent else None
         try:
@@ -119,6 +143,19 @@ class ContainerClone(EntityAdapter):
     rtypes_to_skip = set()
     etypes_to_skip = set()
 
+    # backward compat property, should be overrided by class attribute when
+    # necessary
+    @property
+    def compulsory_hooks_categories(self):
+        try:
+            categories = self.entity.compulsory_hooks_categories
+            warn('[container 2.4] compulsory_hooks_categories moved to '
+                 'ContainerClone adapter',
+                 DeprecationWarning)
+            return categories
+        except AttributeError:
+            return ()
+
     # These two unimplemented properties are bw compat
     # to drive users from entity.clone_(e/r)types_to_skip
     # to adapter.(e/r)types_to_skip
@@ -242,7 +279,7 @@ class ContainerClone(EntityAdapter):
                  DeprecationWarning)
             return self.entity._complete_rql(etype)
         return 'Any X WHERE X is %s, X %s C, C eid %%(container)s' % (
-            etype, self.entity.container_rtype)
+            etype, self.entity.container_config.rtype)
 
     @cachedproperty
     def _no_copy_meta(self):
@@ -309,13 +346,10 @@ class ContainerClone(EntityAdapter):
         return list(self.clonable_etypes())
 
     def clonable_etypes(self):
-        for etype in ordered_container_etypes(self._cw.vreg.schema,
-                                              self.entity.__regid__,
-                                              self.entity.container_rtype,
-                                              self.rtypes_to_skip,
-                                              self.etypes_to_skip,
-                                              self.entity.container_subcontainers):
-            yield etype
+        cfg = self.entity.container_config
+        for etype in self._ordered_container_etypes():
+            if etype not in self.etypes_to_skip:
+                yield etype
 
     def _etype_clone(self, etype, orig_to_clone):
         # 1/ fetch all <etype> entities in current container
@@ -401,7 +435,7 @@ class ContainerClone(EntityAdapter):
         """ Tells whether all targets in the (etype, rtype, TARGET)
         relation can have been already cloned """
         # a shortcut for the container relation
-        if rtype == self.entity.container_rtype:
+        if rtype == self.entity.container_config.rtype:
             return True
         schema = self._cw.vreg.schema
         ordered_etypes = self._ordered_etypes
@@ -554,12 +588,10 @@ class ContainerClone(EntityAdapter):
         return relations
 
     def container_rtypes_etypes(self):
-        etype = self.entity.e_schema.type
-        containerclass = self._cw.vreg['etypes'].etype_class(etype)
-        rtypes, etypes = container_rtypes_etypes(self._cw.vreg.schema, etype,
-                                                 containerclass.container_rtype,
-                                                 skiprtypes=containerclass.container_skiprtypes)
-        return rtypes, etypes
+        schema = self._cw.vreg.schema
+        cfg = self.entity.container_config
+        rtypes, etypes = cfg.structure(schema)
+        return rtypes.union(cfg.inner_relations(schema)), etypes
 
     @cachedproperty
     def _specially_handled_rtypes(self):
@@ -573,12 +605,99 @@ class ContainerClone(EntityAdapter):
     def handle_special_relations(self, deferred_relations):
         pass
 
+    def _ordered_container_etypes(self):
+        """Return list of etypes of a container by dependency order this is
+        provided for simplicity and backward compatibility reasons etypes that
+        are parts of a cycle are undiscriminately added at the end
+        """
+        orders, etype_map = self._container_etype_orders()
+        total_order = []
+        for order in orders:
+            total_order += order
+        return total_order + etype_map.keys()
+
+    def _container_etype_orders(self):
+        """ computes linearizations and cycles of etypes within a container """
+        cfg = self.entity.container_config
+        schema = self._cw.vreg.schema
+        etypes = cfg.structure(schema)[1]
+        orders = []
+        etype_map = dict((etype, _needed_etypes(schema, etype, cfg.etype,
+                                                cfg.rtype, cfg.skiprtypes))
+                         for etype in etypes)
+        maplen = len(etype_map)
+        while etype_map:
+            neworder = _linearize(etype_map, etypes)
+            if neworder:
+                orders.append(neworder)
+            if maplen == len(etype_map):
+                break
+            maplen = len(etype_map)
+        return orders, etype_map
+
 
 
 class MultiParentProtocol(EntityAdapter):
     __regid__ = 'container.multiple_parents'
-    __select__ = yet_unset()
+    __abstract__ = True
 
     def possible_parent(self, rtype, eid):
         pass
 
+
+# more internals ###############################################################
+
+def _needed_etypes(schema, etype, cetype, crtype, computed_rtypes=()):
+    """ finds all container etypes this one depends on to be built
+    start from all subject + object relations """
+    etypes = defaultdict(list)
+    skipetypes = set((cetype,))
+    # these should include rtypes
+    # that create cycles but actually are false dependencies
+    skiprtypes = set(computed_rtypes).union((crtype, 'container_etype', 'container_parent'))
+    skiprtypes.add(crtype)
+    eschema = schema[etype]
+    adjacent_rtypes = [(rschema.type, role)
+                       for role in ('subject', 'object')
+                       for rschema in getattr(eschema, '%s_relations' % role)()
+                       if not (rschema.meta or rschema.final or rschema.type in skiprtypes)]
+    children_rtypes = [r.type for r in children_rschemas(eschema)]
+    parent_etypes = set(parent_eschemas(eschema))
+    for rtype, role in adjacent_rtypes:
+        if rtype in children_rtypes:
+            continue
+        rdef = eschema.rdef(rtype, role)
+        target = getattr(rdef, neg_role(role))
+        if target.type in skipetypes:
+            continue
+        if target.type not in parent_etypes:
+            if rdef.cardinality[0 if role == 'subject' else 1] in '?*':
+                continue
+        etypes[target.type].append((rdef, role))
+    return etypes
+
+def _linearize(etype_map, all_etypes):
+    # Kahn 1962
+    sorted_etypes = []
+    independent = set()
+    for etype, deps in etype_map.items():
+        if not deps:
+            independent.add(etype)
+            del etype_map[etype]
+        for depetype in deps:
+            if depetype not in all_etypes:
+                # out of container dependencies must be added
+                # to complete the graph
+                etype_map[depetype] = dict()
+    while independent:
+        indep_etype = min(independent) # get next in ascii order
+        independent.remove(indep_etype)
+        sorted_etypes.append(indep_etype)
+        for etype, incoming in etype_map.items():
+            if indep_etype in incoming:
+                incoming.pop(indep_etype)
+            if not incoming:
+                independent.add(etype)
+                etype_map.pop(etype)
+    return [etype for etype in sorted_etypes
+            if etype in all_etypes]
