@@ -1,4 +1,4 @@
-# copyright 2011-2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# copyright 2011-2014 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 # contact http://www.logilab.fr -- mailto:contact@logilab.fr
 #
 # This program is free software: you can redistribute it and/or modify it under
@@ -13,16 +13,16 @@
 #
 # You should have received a copy of the GNU Lesser General Public License along
 # with this program. If not, see <http://www.gnu.org/licenses/>.
-
 """cubicweb-container specific hooks and operations"""
+
 from collections import defaultdict
+
 from logilab.common.deprecation import class_deprecated
-from logilab.common.registry import Predicate
 
 from cubicweb import ValidationError
-from cubicweb.server.hook import Hook, DataOperationMixIn, Operation, match_rtype
+from cubicweb.server.hook import Hook, DataOperationMixIn, Operation
 
-from cubes.container.utils import yet_unset, parent_rschemas
+from cubes.container.utils import parent_rschemas
 
 def eid_etype(session, eid):
     return session.describe(eid)[0]
@@ -55,10 +55,17 @@ def _set_container_parent(session, rtype, eid, peid):
             session.warning('relinking %s (eid:%s parent:%s)', rtype, eid, peid)
             return
         # this is a replacement: we allow replacing within the same container
-        #                        for the same rtype
+        # for the same rtype
         old_rtype = find_valued_parent_rtype(target)
         assert old_rtype
-        container = target.cw_adapt_to('Container').related_container
+        try:
+            icontainer = target.cw_adapt_to('Container')
+        except Exception:
+            # select ambiguity, only in debug mode
+            msg = (session._('%s is already in a container through %s') %
+                   (target.e_schema, rtype))
+            raise ValidationError(target.eid, {rtype: msg})
+        container = icontainer.related_container
         parent = session.entity_from_eid(peid)
         parent_container = parent.cw_adapt_to('Container').related_container
         if container.eid != parent_container.eid or old_rtype != rtype:
@@ -92,6 +99,35 @@ class SetContainerRelation(Hook):
             _set_container_parent(self._cw, self.rtype, eid, peid)
 
 
+class SetChildContainerRelation(Hook):
+    """Handle pre-existing subgraphs of the container being attached to the
+    container so that children of these subgraphs have the container relation
+    set."""
+    __regid__ = 'container.set_child_container_relation'
+    __abstract__ = True
+    events = ('after_add_relation',)
+    category = 'container'
+    # __select__ = match_rtype(container.container_rtype)
+
+    def __call__(self):
+        container = self._cw.entity_from_eid(self.eidto)
+        peid = self.eidfrom
+        etypefrom = eid_etype(self._cw, peid)
+        eschema = self._cw.vreg.schema[etypefrom]
+        for rschema, targets, role in eschema.relation_definitions():
+            if rschema in container.container_config.skiprtypes:
+                continue
+            rdef = eschema.rdef(rschema, role, takefirst=True)
+            if rdef.composite == role:
+                if role == 'subject':
+                    rql = 'Any C WHERE X %s C, X eid %%(x)s' % rschema
+                else:
+                    rql = 'Any C WHERE C %s X, X eid %%(x)s' % rschema
+                for eid, in self._cw.execute(rql, {'x': peid}):
+                    AddContainerRelationOp.get_instance(self._cw).add_data(
+                        (eid, peid))
+
+
 class SetContainerParent(Hook):
     __metaclass__ = class_deprecated
     __deprecation_warning__ = ('[2.2.0] SetContainerParent is deprecated, '
@@ -135,10 +171,12 @@ class AddContainerRelationOp(DataOperationMixIn, Operation):
             cprotocol = parent.cw_adapt_to('Container')
             container = cprotocol.related_container
             if container is None:
-                self.critical('container entity could not be reached from %s, '
-                              'you may have ordering issues', parent)
+                # A subgraph of the container can exist without a container.
+                self.warning('Cannot find container related to parent entity %d; '
+                             'probably because it is part of a subgraph not yet '
+                             'related to its container.' % eid)
                 continue
-            container_rtype_rel[container.container_rtype].append((eid, container.eid))
+            container_rtype_rel[container.container_config.rtype].append((eid, container.eid))
             container_etype_rel.append((eid, self._container_cwetype_eid(container, cwetype_eid_map)))
         if container_rtype_rel:
             session.add_relations(container_rtype_rel.items())
@@ -151,7 +189,8 @@ class AddContainerRelationOp(DataOperationMixIn, Operation):
 class CloneContainer(Hook):
     __regid__ = 'container.clone'
     events = ('after_add_relation',)
-    __select__ = yet_unset() # use match_rtype(container_clone_rtype)
+    #__select__ = None use match_rtype(container_clone_rtype)
+    __abstract__ = True
     category = 'container'
 
     def __call__(self):
@@ -171,9 +210,12 @@ class CloneContainerOp(DataOperationMixIn, Operation):
         for cloneid in self.get_data():
             with self.session.repo.internal_session() as session:
                 cloned = session.entity_from_eid(cloneid)
-                with session.deny_all_hooks_but(*cloned.compulsory_hooks_categories):
+                cloner = cloned.cw_adapt_to('Container.clone')
+                # XXX(syt) why isn't this in the adapter rather than in the
+                # operation?
+                with session.deny_all_hooks_but(*cloner.compulsory_hooks_categories):
                     self.prepare_cloned_container(session, cloned)
-                    cloned.cw_adapt_to('Container.clone').clone()
+                    cloner.clone()
                     self.finalize_cloned_container(session, cloned)
                     session.commit()
 
